﻿using CarpentryWorkshopContracts.BindingModels;
using CarpentryWorkshopContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CarpentryWorkshopView
{
	public partial class FormMain : Form
	{
		private readonly ILogger _logger;
		private readonly IOrderLogic _orderLogic;
		private readonly IReportLogic _reportLogic;
		private readonly IWorkProcess _workProcess;

		public FormMain(ILogger<FormMain> logger, IOrderLogic orderLogic, IReportLogic reportLogic, IWorkProcess workProcess)
		{
			InitializeComponent();
			_logger = logger;
			_orderLogic = orderLogic;
			_reportLogic = reportLogic;
			_workProcess = workProcess;
		}
		private void FormMain_Load(object sender, EventArgs e)
		{
			LoadData();
		}
		private void LoadData()
		{
			_logger.LogInformation("Загрузка заказов");
			// прописать логику
			try
			{
				var list = _orderLogic.ReadList(null);
				if (list != null)
				{
					dataGridView.DataSource = list;
					dataGridView.Columns[1].Visible = false;
					dataGridView.Columns[3].Visible = false;
					dataGridView.Columns[4].Visible = false;

				}
				_logger.LogInformation("Загрузка продуктов");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки продуктов");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
		private void КомпонентыToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormComponents));
			if (service is FormComponents form)
			{
				form.ShowDialog();
			}
		}
		private void ИзделияToolStripMenuItem_Click(object sender, EventArgs e)
		{
			// прописать логику
			var service = Program.ServiceProvider?.GetService(typeof(FormWoods));
			if (service is FormWoods form)
			{
				form.ShowDialog();
			}
		}
		private void ButtonCreateOrder_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormCreateOrder));
			if (service is FormCreateOrder form)
			{
				form.ShowDialog();
				LoadData();
			}
		}
		private void ButtonTakeOrderInWork_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation("Заказ №{id}. Меняется статус на 'В работе'", id);
				try
				{
					var operationResult = _orderLogic.TakeOrderInWork(new OrderBindingModel { Id = id });
					if (!operationResult)
					{
						throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
					}
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка передачи заказа в работу");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
		}
		private void ButtonOrderReady_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation("Заказ №{id}. Меняется статус на 'Готов'", id);
				try
				{
					var operationResult = _orderLogic.FinishOrder(new OrderBindingModel { Id = id });
					if (!operationResult)
					{
						throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
					}
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка отметки о готовности заказа"); MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
		}
		private void ButtonIssuedOrder_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation("Заказ №{id}. Меняется статус на 'Выдан'", id);
				try
				{
					var operationResult = _orderLogic.DeliveryOrder(new OrderBindingModel { Id = id });
					if (!operationResult)
					{
						throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
					}
					_logger.LogInformation("Заказ №{id} выдан", id);
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка отметки о выдачи заказа");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
		}
		private void ButtonRef_Click(object sender, EventArgs e)
		{
			LoadData();
		}

		private void списокИзделийToolStripMenuItem_Click(object sender, EventArgs e)
		{
			using var dialog = new SaveFileDialog { Filter = "docx|*.docx" };
			if (dialog.ShowDialog() == DialogResult.OK)
			{
				_reportLogic.SaveComponentsToWordFile(new ReportBindingModel { FileName = dialog.FileName });
				MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}

		private void изделияПоКомпонентамToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormReportWoodComponents));
			if (service is FormReportWoodComponents form)
			{
				form.ShowDialog();
			}
		}

		private void списокЗаказовToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormReportOrders));
			if (service is FormReportOrders form)
			{
				form.ShowDialog();
			}
		}

		private void клиентыToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormClients));
			if (service is FormClients form)
			{
				form.ShowDialog();
			}
		}
		private void запускРаботToolStripMenuItem_Click(object sender, EventArgs e)
		{
			_workProcess.DoWork((Program.ServiceProvider?.GetService(typeof(IImplementerLogic)) as IImplementerLogic)!, _orderLogic);
			MessageBox.Show("Процесс обработки запущен", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
		}
		private void исполнителиToolStripMenuItem_Click_1(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormImplementers));
			if (service is FormImplementers form)
			{
				form.ShowDialog();
			}
		}
	}
}
